package com.mortals.xhx.module.matter.service.impl;

import cn.hutool.core.util.ReflectUtil;
import cn.hutool.setting.Setting;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.base.framework.config.InterceptorConfig;
import com.mortals.xhx.common.code.FiletypeEnum;
import com.mortals.xhx.common.code.SourceEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.common.utils.MatterDetailHtmlParseUtil;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.matter.dao.MatterDao;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.model.vo.MatterInfo;
import com.mortals.xhx.module.matter.service.MatterDatumFileService;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import com.mortals.xhx.module.matter.service.MatterExtService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.model.SiteMatterQuery;
import com.mortals.xhx.module.site.service.SiteMatterService;
import com.mortals.xhx.module.site.service.SiteService;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * MatterService
 * 基础事项 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("matterService")
@Slf4j
public class MatterServiceImpl extends AbstractCRUDServiceImpl<MatterDao, MatterEntity, Long> implements MatterService {
    @Autowired
    private SiteMatterService siteMatterService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private InterceptorConfig interceptorConfig;
    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private MatterDatumFileService matterDatumFileService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private MatterExtService matterExtService;
//    @Autowired
//    private SiteThemeMatterService siteThemeMatterService;
//    @Autowired
//    private SiteThemeService siteThemeService;
//    @Autowired
//    private WindowMatterService windowMatterService;


    @Override
    protected void saveBefore(MatterEntity entity, Context context) throws AppException {
        //判断如果是自定义事项，需要更新区域编码
        if (SourceEnum.自定义.getValue().equals(entity.getSource())) {
            if (!ObjectUtils.isEmpty(entity.getSiteId())) {
                SiteEntity siteEntity = siteService.get(entity.getSiteId(), context);
                entity.setAreaCode(siteEntity.getAreaCode());
                DeptEntity extCache = deptService.getExtCache(entity.getDeptCode());
                entity.setDeptName(extCache == null ? "" : extCache.getName());
            }

        }
        super.saveBefore(entity, context);
    }

    @Override
    protected void updateBefore(MatterEntity entity, Context context) throws AppException {
        if (SourceEnum.自定义.getValue().equals(entity.getSource())) {
            if (!ObjectUtils.isEmpty(entity.getSiteId())) {
                SiteEntity siteEntity = siteService.get(entity.getSiteId(), context);
                entity.setAreaCode(siteEntity.getAreaCode());
                DeptEntity extCache = deptService.getExtCache(entity.getDeptCode());
                entity.setDeptName(extCache == null ? "" : extCache.getName());
            }
            MatterEntity beforeMatterEntity = this.get(entity.getId(), context);
            //如果部门切换了 级联修改关联的其它项
            if (!ObjectUtils.isEmpty(entity.getDeptCode())) {
                if (!entity.getDeptCode().equals(beforeMatterEntity.getDeptCode())) {
                    //更新站点事项关联项
                    SiteMatterQuery condition = new SiteMatterQuery();
                    condition.setMatterId(entity.getId());
                    SiteMatterQuery siteMatterQuery = new SiteMatterQuery();
                    DeptEntity extCache = deptService.getExtCache(entity.getDeptCode());

                    siteMatterQuery.setDeptId(extCache == null ? 0L : extCache.getId());
                    siteMatterQuery.setDeptCode(entity.getDeptCode());
                    siteMatterQuery.setDeptName(entity.getDeptName());
                    siteMatterService.updateBatch(siteMatterQuery, condition, context);

                    //更新站点窗口事项关联项
//                    WindowMatterQuery winCondition = new WindowMatterQuery();
//                    winCondition.setSiteMatterId(entity.getId());
//                    WindowMatterQuery windowMatterQuery = new WindowMatterQuery();
//                    windowMatterQuery.setDeptId(extCache == null ? 0L : extCache.getId());
//                    windowMatterQuery.setDeptCode(entity.getDeptCode());
//                    windowMatterQuery.setDeptName(entity.getDeptName());
//                    windowMatterService.updateBatch(windowMatterQuery, winCondition, context);
                }
            }

        }
        super.updateBefore(entity, context);
    }


    @Override
    protected void updateAfter(MatterEntity entity, Context context) throws AppException {
        //同步更新所有站点
        List<SiteMatterEntity> collect = siteMatterService.find(new SiteMatterQuery().matterId(entity.getId())).stream()
                .map(item -> {
                    item.setMatterName(entity.getMatterName());
                    return item;
                }).collect(Collectors.toList());
        siteMatterService.update(collect, context);
        super.updateAfter(entity, context);
    }


    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        //有子节点 禁止删除
        super.removeBefore(ids, context);
    }

    @Override
    public Rest<String> addMatterToSite(String matterIds, Long siteId, Context context) {
        if (ObjectUtils.isEmpty(siteId)) {
            throw new AppException("请选择对应站点");
        }
        List<Long> matterIdList = Arrays.asList(matterIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        MatterQuery matterQuery = new MatterQuery();
        matterQuery.setIdList(matterIdList);
        List<MatterEntity> matterEntities = this.find(matterQuery);
        int success = 0;
        int fail = 0;
        List<SiteMatterEntity> siteMatterEntities = new ArrayList<>();

        for (MatterEntity matterEntity : matterEntities) {
            SiteMatterEntity siteMatterEntity = updateOrSave(matterEntity, siteId, context);
            if (!ObjectUtils.isEmpty(siteMatterEntity)) {
                siteMatterEntities.add(siteMatterEntity);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(siteMatterEntities)) {
            siteMatterService.save(siteMatterEntities, context);
        }
        String msg = "当前加入事项已存在！";
        if (matterIdList.size() == 1) {
            if (success > 0) {
                msg = "加入事项成功！";
            }
        } else if (matterIdList.size() > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入事项成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入事项成功%d条，重复加入事项%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入事项%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private SiteMatterEntity updateOrSave(MatterEntity item, Long siteId, Context context) {
        SiteMatterEntity siteMatterEntity = siteMatterService.selectOne(new SiteMatterQuery().siteId(siteId).matterId(item.getId()));
        if (ObjectUtils.isEmpty(siteMatterEntity)) {
            SiteEntity siteEntity = siteService.getCache(siteId.toString());
            Rest<SiteMatterEntity> rest = switchMatterToSiteMatter(item, siteEntity, context);
            if (rest.getCode() == YesNoEnum.YES.getValue()) {
                return rest.getData();
            }
        }
        return null;
    }

    @Override
    public Rest<SiteMatterEntity> switchMatterToSiteMatter(MatterEntity item, SiteEntity siteEntity, Context context) {
        try {
            DeptEntity deptEntity = deptService.getExtCache(item.getDeptCode());
            SiteMatterEntity siteMatterEntity = new SiteMatterEntity();
            //SiteEntity siteEntity = siteService.get(siteId);
            siteMatterEntity.setSiteId(siteEntity.getId());
            siteMatterEntity.setSiteName(siteEntity.getSiteName());
            siteMatterEntity.setMatterId(item.getId());
            siteMatterEntity.setMatterCode(item.getMatterNo());
            siteMatterEntity.setMatterName(item.getMatterName());
            siteMatterEntity.setDeptCode(item.getDeptCode());
            siteMatterEntity.setSource(item.getSource());
            siteMatterEntity.setEventTypeShow(item.getEventTypeShow());
            siteMatterEntity.setDeptName(deptEntity == null ? "" : deptEntity.getName());
            siteMatterEntity.setDeptId(deptEntity == null ? -1L : deptEntity.getId());
            siteMatterEntity.setCreateUserId(context == null ? 1L : context.getUser() == null ? 1L : context.getUser().getId());
            siteMatterEntity.setCreateTime(new Date());
            return Rest.ok(siteMatterEntity);

        } catch (Exception e) {
            log.error("转换异常", e);
            return Rest.fail(e.getMessage());
        }
    }

    @Override
    public Result<MatterEntity> findSubList(MatterEntity matterQuery, PageInfo pageInfo, Context context) throws AppException {
        SiteEntity siteCache = siteService.getCache(matterQuery.getSiteId().toString());
        if (ObjectUtils.isEmpty(matterQuery.getAreaCode())) {
            matterQuery.setAreaCode(siteCache == null ? null : siteCache.getAreaCode());
        }
        return this.dao.getSubList(matterQuery, pageInfo);
    }

    @Override
    public Result<MatterInfo> findMicroList(MatterQuery matterQuery, PageInfo pageInfo, Context context) throws AppException {
        Result<MatterInfo> matterInfoResult = new Result<>();
        Result<MatterEntity> matterEntityResult = this.find(matterQuery, pageInfo, context);
        List<MatterInfo> matterInfoList = matterEntityResult.getList().stream().map(item -> {
            MatterInfo matterInfo = new MatterInfo();
            BeanUtils.copyProperties(item, matterInfo, BeanUtil.getNullPropertyNames(item));
            return matterInfo;
        }).collect(Collectors.toList());
        matterInfoResult.setList(matterInfoList);
        matterInfoResult.setPageInfo(pageInfo);

        return matterInfoResult;
    }

    @Override
    public Rest<String> buildMatterDetail(MatterEntity matterEntity, Context context) {
        //根据事项url 获取事项详细信息，构建相关详细信息
        String html = null;
        try {
            Document dom = Jsoup.connect(matterEntity.getUrl()).get();
            // html = Jsoup.connect(matterEntity.getUrl()).get().body().html();
            // html = HttpUtil.get(matterEntity.getUrl());
            //System.out.println(html);
            // log.info("build detail:" + matterEntity.getMatterNo());
            //设置基本属性值
            Setting baseInfoSetting = interceptorConfig.getBaseInfoSetting();
            Setting sqclInfoSetting = interceptorConfig.getSqclInfoSetting();

            //更新部门信息
            DeptEntity extCache = deptService.getExtCache(matterEntity.getDeptCode());
            matterEntity.setDeptName(extCache == null ? "" : extCache.getName());

/*            if(!ObjectUtils.isEmpty(matterEntity.getDeptCode())&&ObjectUtils.isEmpty(matterEntity.getDeptName())){
                DeptEntity extCache = deptService.getExtCache(matterEntity.getDeptCode());
                matterEntity.setDeptName(extCache==null?"":extCache.getName());
            }*/
            //构建基础信息参数
            savebaseInfo(matterEntity, dom, baseInfoSetting);

            //更新材料属性
            saveDatumInfo(matterEntity, context, dom, sqclInfoSetting);

            //更新事项更新标志
            matterEntity.setHaveGetMatterInfo("true");

        } catch (Exception e) {
            log.error(String.format("获取详细异常! id:%s,matterCode:%s,url:%s", matterEntity.getId(), matterEntity.getMatterNo(), matterEntity.getUrl()), e);
            return Rest.fail("更新事项失败！" + e.getMessage());
        }
        return Rest.ok("更新详细成功！");
    }




    private void savebaseInfo(MatterEntity matterEntity, Document dom, Setting baseInfoSetting) {
        Map<String, String> baseInfoMap = MatterDetailHtmlParseUtil.getbaseInfoMapByHtml(dom);
        List<MatterExtEntity> matterExtEntities = new ArrayList<>();
        long sortN = 1;
        for (Map.Entry<String, String> m : baseInfoMap.entrySet()) {
            String value = baseInfoSetting.getOrDefault(m.getKey(), "");
            if (!ObjectUtils.isEmpty(value)) {
                if (!ObjectUtils.isEmpty(m.getValue()) && !"无".equals(m.getValue())) {
                    try {
                        ReflectUtil.setFieldValue(matterEntity, value, m.getValue());
                    } catch (Exception e) {
                        log.error(String.format("设置值异常！key:%s,value:%s,orginKey:%s", value, m.getValue(), m.getKey()));
                    }
                }
            } else {
                //未配置的属性映射添加到扩展表中
                //属性保存到扩展表中
                MatterExtEntity matterExtEntity = new MatterExtEntity();
                matterExtEntity.initAttrValue();
                matterExtEntity.setMatterId(matterEntity.getId());
                matterExtEntity.setMatterCode(matterEntity.getMatterNo());
                matterExtEntity.setFieldName(m.getKey());
                matterExtEntity.setFieldValue(m.getValue());
                matterExtEntity.setFieldOrderNo(sortN);
                matterExtEntity.setSource(SourceEnum.政务网.getValue());
                matterExtEntity.setCreateTime(new Date());
                matterExtEntity.setCreateUserId(1L);
                matterExtEntities.add(matterExtEntity);
                sortN++;
            }
        }

        if (!ObjectUtils.isEmpty(matterExtEntities)) {
            matterExtService.save(matterExtEntities, null);
        }

    }


    private void saveDatumInfo(MatterEntity matterEntity, Context context, Document dom, Setting sqclInfoSetting) {
        Long[] datumIds = matterDatumService.find(new MatterDatumQuery().matterId(matterEntity.getId()).source(SourceEnum.政务网.getValue())).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(datumIds)) {
            matterDatumService.remove(datumIds, context);
            Long[] datumFileIds = matterDatumFileService.find(new MatterDatumFileQuery().datumIdList(Arrays.asList(datumIds))).stream().map(item -> item.getId()).toArray(Long[]::new);
            if (!ObjectUtils.isEmpty(datumFileIds)) {
                matterDatumService.remove(datumFileIds, context);
            }
        }

        List<Map<String, Object>> sqclList = MatterDetailHtmlParseUtil.getsqclInfoMapByHtml(dom);
        for (Map<String, Object> sqclMap : sqclList) {
            Map<String, String> baseinfoMap = (HashMap<String, String>) sqclMap.get("baseinfo");
            List<MatterDatumFileEntity> blankList = (List<MatterDatumFileEntity>) sqclMap.get("blankList");
            List<MatterDatumFileEntity> sampleList = (List<MatterDatumFileEntity>) sqclMap.get("sampleList");

            MatterDatumEntity datumEntity = new MatterDatumEntity();
            datumEntity.initAttrValue();
            datumEntity.setMatterId(matterEntity.getId());
            datumEntity.setMatterCode(matterEntity.getMatterNo());
            datumEntity.setMaterialName(matterEntity.getMatterName());
            datumEntity.setSource(SourceEnum.政务网.getValue());
            datumEntity.setCreateTime(new Date());
            datumEntity.setCreateUser("system");
            datumEntity.setCreateUserId(1L);
            //材料详情
            baseinfoMap.entrySet().stream().forEach(m -> {
                String value = sqclInfoSetting.getOrDefault(m.getKey(), "");
                if (!ObjectUtils.isEmpty(value)) {
                    try {
                        ReflectUtil.setFieldValue(datumEntity, value, m.getValue());
                    } catch (Exception e) {
                        log.error(String.format("设置值异常！key:%s,value:%s", value, m.getValue()));
                    }
                }
            });
            matterDatumService.save(datumEntity);

            //示例样表
            if (!ObjectUtils.isEmpty(sampleList)) {
                sampleList.forEach(fileEntity -> {
                    fileEntity.setDatumId(datumEntity.getId());
                    fileEntity.setMatterCode(matterEntity.getMatterNo());
                    fileEntity.setMaterialName(datumEntity.getMaterialName());
                });
            }
            matterDatumFileService.save(sampleList);

            //空白样表
            if (!ObjectUtils.isEmpty(blankList)) {
                blankList.forEach(fileEntity -> {
                    fileEntity.setDatumId(datumEntity.getId());
                    fileEntity.setMatterCode(matterEntity.getMatterNo());
                    fileEntity.setMaterialName(datumEntity.getMaterialName());
                });
            }
            matterDatumFileService.save(blankList);
        }
    }


}